/* =============================================================================
 UPC - EETAC - CSD - http://digsys.cat

Example of using C language and PIC microcontrollers to solve logic equations or
design Combinational blocks.

A Dual_MUX4 combinational circuit to be used to learn about how to set the
 digital input and outputs of the microcontroller

The circuit is adapted from a previuous PIC18F4520 version available in:
http://digsys.upc.es/ed//CSD/terms/1112Q2/EX/EX4/Dual_MUX4_PIC18F4520.zip  
   ========================================================================== */

#include <xc.h>	
   /* This general header file includes the device specific headers like
    "pic16f877a.h", <pic16f877a.h>. This file contains all the microcontroller 
    declarations. All the header files are located in the C compilers's specific
    of: C:\Program Files (x86)\Microchip\xc8\v1.33\include\pic16f877a.h */

/* ------------------------------------
	Definitions 
   -------------------------------------------------------------------------- */
//#define 	PORTBIT(adr, bit) ( (unsigned)(&adr)*8 + (bit) )
 /* This structure used in some books and references. It allows accessing
to a given bit into an 8-bit register. But it's not going to be used in CSD. 
We'll better make use of bitwise opertions only in the hardware-specific 
functions: read_inputs() and write_outputs()*/
	
/* ------------------------------------
	Function prototypes 
   -------------------------------------------------------------------------- */
void init_system (void);
void read_inputs (void);
void truth_table (void);
void write_outputs (void);

/* -----------------------------------
	Global variables 
   -------------------------------------------------------------------------- */
/* NOT USED in CSD: 	static bit S3 @ PORTBIT(PORTA, 5), etc. 
	the option of accessing every individual bit	*/ 

/* Let's use convenient byte-type (char) variables. Let's say that we like
 to use similar names for our combinational circuit, as in previous chapters. */

static char Enable;
static char Channel_0; 
static char Channel_1;
static char Channel_2;
static char Channel_3;
static char Sel;
static char Y_out;  

static char Data_in; // A convenient 8-bit buffer where to store parcial results

/* =============================================================================
		Main function 
   ========================================================================== */
void main(void){    // Let's make the main() the simplest possible.
    init_system();
    while(1) {		/* loop forever reading inputs and calculating outputs*/
	read_inputs();
        truth_table();
        write_outputs();
    }		
}	

/* ------------------------------------
	Function definitions 
   -------------------------------------------------------------------------- */

/* =============================================================================
 Scan inputs and process the bits so that we can set our convenient variables 
 many of the variables are "char" because we can easily monitor them using
 the watch window for debugging purposes.
  =========================================================================== */
void read_inputs (void){         
        				// not used here: Enable = Enable_pin;

    // Let's prepare our Enable bit as a byte of memory,
    // so that will be able to "watch" it
    Data_in = PORTA;    // Read all the bits (like LD a parellel 8-bit register)
    Data_in = Data_in & 0b00100000;   // mask the bit of interest (bitwise AND)
    Enable = Data_in >> 5;   // But wise shift right 5 bits
        // The three operation can be placed in a single line of C:
          //  Enable = (PORTA & 0b00100000) >> 5;
    
    Data_in = PORTC;    // Read or poll all the PORTC bits in a single statement
                    // (equivalent to a parallel LD operation of 8-bit data)
    // Let's process the channels using a mask and shift operators
    Channel_0 = Data_in& 0b00000011;
    Channel_1 = (Data_in & 0b00001100)>> 2;
    Channel_2 = (Data_in & 0b00110000)>> 4;
    Channel_3 = (Data_in & 0b11000000)>> 6;

    // Let's read the Sel variable
    Sel = (PORTD & 0b11000000)>> 6;
}

/* =============================================================================
 The behavioural description of the circuit operation or the data processing
 algorithm.
 When this function is executed, all the variables are in the software domain,
 meaning that they are completelly independent of the hardware, thus the
 algorithmn is compatible among microcontroller families and pin connexions.
  =========================================================================== */
void truth_table(void){

// This is the combinational circuit truth table described in a behavioural way:

	if (Sel == 0 && Enable == 1) {
		Y_out = Channel_0;
		}
	else if (Sel == 1 && Enable == 1){
		Y_out = Channel_1;
		}
	else if (Sel == 2 && Enable == 1){
		Y_out = Channel_2;
		}
	else if (Sel == 3 && Enable == 1){
		Y_out = Channel_3;
        }
	else 
	    Y_out = 0x00;  
}   

/* =============================================================================
	Driving the output pins. Let's drive the output ports, converting 
	microcontroller variables into electrical signals.    
   ========================================================================== */
void write_outputs(void){
	// Here it is easy, bacause the other port signals are unused. 
	PORTD = Y_out;
 // But generally, it will be required to mask and organise the port pins as
 // when reading inputs (Add an example how to do it)
}    

/* =============================================================================
	Init the microcontroller 
   ========================================================================== */
   
void init_system (void){ 

 /* These are the PIC16F877A configuration bits translated to the XC8 Compiler.
    They can be generated automatically in the MPLAB X IDE, but normally,
    you don't have to change them.
    So, copy and paste them as they are to your projects */

//#pragma config FOSC = EXTRC   // Oscillator Selection bits (RC oscillator)
#pragma config FOSC = XT     	// External XTAL Oscillator

//#pragma config WDTE = ON      // Watchdog Timer Enable bit (WDT enabled)
#pragma config WDTE = OFF       // Watchdog Timer not in use (WDT disabled)

//#pragma config PWRTE = OFF    // Power-up Timer Enable bit (PWRT disabled)
#pragma config PWRTE = ON      	// Power-up Timer Enable bit (PWRT enabled)

#pragma config BOREN = ON       // Brown-out Reset Enable bit (BOR enabled)
#pragma config LVP = ON         // Low-Voltage (Single-Supply) In-Circuit Serial
				// Programming Enable bit (RB3/PGM pin has PGM 
				// function; low-voltage programming enabled)

#pragma config CPD = OFF        // Data EEPROM Memory Code Protection bit 
        			//(Data EEPROM code protection off)
#pragma config WRT = OFF        // Flash Program Memory Write Enable bits (Write 
				// protection off; all program memory may be 
				// written to by EECON control)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit 
                		// (Code protection off)
    
// Inicialise operations starts here:
    // Pay attention to how the ports are configured
    // using mainly PORTx and TRISx registers
	PORTA 	= 0x00;     // Reset all Flip-Flops at PORTA
		
	ADCON1 	= 0x06; /* All the PortA is set to be digital I/O. Remember that
                           the PORTA can be also defined as analogue inputs*/

	TRISA 	= 0b00100000; /* RA5 is an input (the Enable ) */

	PORTD 	= 0x00;       // Reset all Flip-Flops at PORTD
	
	TRISD 	= 0b11000000;	 // PORTD7 and PORTD6 are inputs
                             // PORTD1 and PORTD0 are outputs

	PORTC 	= 0x00;        // Reset all Flip-Flops at PORTC
	
	TRISC 	= 0xFF;         // PORTC are all inputs for the channels

	GIE 	= 0;	// No interrupts in this very simple application 
	
	Y_out = 0x00;	//Output cleared 
	
}
